@@ -1,4 +1,4 @@
-// ichimoku - Window Functions: using an algorithm with the following license:
+// ichimoku - Window Functions: modified from code with the following license:
 
 /*
  Based on http://opensource.org/licenses/MIT
@@ -22,56 +22,73 @@
  */
 
 #include <deque>
-#include <cpp11.hpp>
-
-// types of calculations
-enum CalcType {MIN, MAX};
-
-// function arguments for non-data
-struct Args {
-  int window;
-  CalcType ctype;
-};
-
-// calculates rolling window for {minimum, maximum}
-cpp11::doubles roll_minmax(const cpp11::doubles& x, Args a) {
-
-  int n  = x.size();
-  cpp11::writable::doubles rollx(n);
-
-  std::deque< std::pair<long double, int> > deck;
-  for (int i = 0; i < x.size(); ++i) {
-    if(a.ctype == MIN) {
-      while (!deck.empty() && deck.back().first >= x[i])
-        deck.pop_back();
-    } else {
-      while (!deck.empty() && deck.back().first <= x[i])
-        deck.pop_back();
-    }
-    deck.push_back(std::make_pair(x[i], i));
-
-    while(deck.front().second <= i - a.window)
-      deck.pop_front();
-
-    long double min = deck.front().first;
-    if (i < a.window - 1) {
-      rollx[i] = NA_REAL;
-    } else {
-      rollx[i] = min;
-    }
-  }
-  return rollx;
-}
+#include "cpp11/doubles.hpp"
 
 [[cpp11::register]]
 cpp11::doubles maxOver(const cpp11::doubles& x, int window) {
-  Args a; a.window = window; a.ctype = MAX;
-  return roll_minmax(x, a);
+
+  int n = x.size(), w1 = window - 1;
+  cpp11::writable::doubles vec(n);
+
+  std::deque<std::pair<long double, int>> deck;
+  for (int i = 0; i < n; ++i) {
+      while (!deck.empty() && deck.back().first <= x[i])
+        deck.pop_back();
+    deck.push_back(std::make_pair(x[i], i));
+
+    while(deck.front().second <= i - window)
+      deck.pop_front();
+
+    long double min = deck.front().first;
+    if (i >= w1) {
+      vec[i] = min;
+    } else {
+      vec[i] = NA_REAL;
+    }
+  }
+  return vec;
 }
 
 [[cpp11::register]]
 cpp11::doubles minOver(const cpp11::doubles& x, int window) {
-  Args a; a.window = window; a.ctype = MIN;
-  return roll_minmax(x, a);
+
+  int n = x.size(), w1 = window - 1;
+  cpp11::writable::doubles vec(n);
+
+  std::deque<std::pair<long double, int>> deck;
+  for (int i = 0; i < n; ++i) {
+      while (!deck.empty() && deck.back().first >= x[i])
+        deck.pop_back();
+    deck.push_back(std::make_pair(x[i], i));
+
+    while(deck.front().second <= i - window)
+      deck.pop_front();
+
+    long double min = deck.front().first;
+    if (i >= w1) {
+      vec[i] = min;
+    } else {
+      vec[i] = NA_REAL;
+    }
+  }
+  return vec;
+}
+
+[[cpp11::register]]
+cpp11::doubles meanOver(const cpp11::doubles& x, int window) {
+
+  int n = x.size(), w1 = window - 1;
+  cpp11::writable::doubles vec(n);
+  long double sum = 0;
+  for (int i = 0; i < n; ++i) {
+    sum += x[i];
+    if (i >= w1) {
+      vec[i] = sum / window;
+      sum -= x[i - w1];
+    } else {
+      vec[i] = NA_REAL;
+    }
+  }
+  return vec;
 }
 
